# == Class: kubernetes::node::params
#
# Default parameter values for the kubernetes::node module
#
class kubernetes::node::params {
  # kubelet options
  $kubelet_service_ensure = 'running'
  $kubelet_journald_forward_enable = false
  $kubelet_service_enable = true
  $kubelet_pod_manifest_path_purge = false
  $kubelet_address = '0.0.0.0'
  $kubelet_allow_privileged = false
  $kubelet_anonymous_auth = undef
  $kubelet_api_servers = ['http://127.0.0.1:8080']
  $kubelet_authentication_token_webhook = undef
  $kubelet_authentication_token_webhook_cache_ttl = undef
  $kubelet_authorization_mode = undef
  $kubelet_authorization_webhook_cache_authorized_ttl = undef
  $kubelet_authorization_webhook_cache_unauthorized_ttl = undef
  $kubelet_cadvisor_port = undef
  $kubelet_cert_dir = undef
  $kubelet_cgroup_driver = undef
  $kubelet_cgroup_root = undef
  $kubelet_chaos_chance = undef
  $kubelet_client_ca_file = undef
  $kubelet_cloud_config = undef
  $kubelet_cloud_provider = undef
  $kubelet_cluster_dns = undef
  $kubelet_cluster_domain = undef
  $kubelet_config = undef
  $kubelet_container_runtime = undef
  $kubelet_container_runtime_endpoint = undef
  $kubelet_containerized = undef
  $kubelet_cpu_cfs_quota = undef
  $kubelet_docker_endpoint = undef
  $kubelet_docker_exec_handler = undef
  $kubelet_enable_controller_attach_detach = undef
  $kubelet_enable_custom_metrics = undef
  $kubelet_enable_debugging_handlers = undef
  $kubelet_enable_server = undef
  $kubelet_event_burst = undef
  $kubelet_event_qps = undef
  $kubelet_eviction_hard = undef
  $kubelet_eviction_max_pod_grace_period = undef
  $kubelet_eviction_minimum_reclaim = undef
  $kubelet_eviction_pressure_transition_period = undef
  $kubelet_eviction_soft = undef
  $kubelet_eviction_soft_grace_period = undef
  $kubelet_exit_on_lock_contention = undef
  $kubelet_experimental_allowed_unsafe_sysctls = undef
  $kubelet_experimental_bootstrap_kubeconfig = undef
  $kubelet_experimental_cgroups_per_qos = undef
  $kubelet_experimental_check_node_capabilities_before_mount = undef
  $kubelet_experimental_cri = undef
  $kubelet_experimental_fail_swap_on = undef
  $kubelet_experimental_kernel_memcg_notification = undef
  $kubelet_experimental_mounter_path = undef
  $kubelet_file_check_frequency = undef
  $kubelet_google_json_key = undef
  $kubelet_hairpin_mode = undef
  $kubelet_healthz_bind_address = undef
  $kubelet_healthz_port = undef
  $kubelet_host_ipc_sources = undef
  $kubelet_host_network_sources = undef
  $kubelet_host_pid_sources = undef
  $kubelet_hostname_override = $::fqdn
  $kubelet_http_check_frequency = undef
  $kubelet_image_gc_high_threshold = 90
  $kubelet_image_gc_low_threshold = 80
  $kubelet_image_service_endpoint = undef
  $kubelet_iptables_drop_bit = undef
  $kubelet_iptables_masquerade_bit = undef
  $kubelet_kube_api_burst = undef
  $kubelet_kube_api_content_type = undef
  $kubelet_kube_api_qps = undef
  $kubelet_kube_reserved = undef
  $kubelet_kubeconfig = undef
  $kubelet_kubelet_cgroups = undef
  $kubelet_lock_file = undef
  $kubelet_low_diskspace_threshold_mb = 256
  $kubelet_make_iptables_util_chains = undef
  $kubelet_manifest_url = undef
  $kubelet_manifest_url_header = undef
  $kubelet_master_service_namespace = undef
  $kubelet_max_open_files = 1000000
  $kubelet_max_pods = 110
  $kubelet_minimum_image_ttl_duration = '2m0s'
  $kubelet_node_ip = undef
  $kubelet_node_labels = undef
  $kubelet_node_status_update_frequency = '10s'
  $kubelet_non_masquerade_cidr = undef
  $kubelet_oom_score_adj = undef
  $kubelet_outofdisk_transition_frequency = '5m0s'
  $kubelet_pod_cidr = undef
  $kubelet_pod_infra_container_image = undef
  $kubelet_pod_manifest_path = undef
  $kubelet_pods_per_core = undef
  $kubelet_port = 10250
  $kubelet_protect_kernel_defaults = undef
  $kubelet_register_node = true
  $kubelet_register_schedulable = undef
  $kubelet_registry_burst = undef
  $kubelet_registry_qps = undef
  $kubelet_require_kubeconfig = undef
  $kubelet_resolv_conf = undef
  $kubelet_rkt_api_endpoint = undef
  $kubelet_rkt_path = undef
  $kubelet_root_dir = undef
  $kubelet_runonce = undef
  $kubelet_runtime_cgroups = undef
  $kubelet_runtime_request_timeout = undef
  $kubelet_seccomp_profile_root = undef
  $kubelet_serialize_image_pulls = true
  $kubelet_streaming_connection_idle_timeout = undef
  $kubelet_sync_frequency = undef
  $kubelet_system_cgroups = undef
  $kubelet_system_reserved = undef
  $kubelet_tls_cert_file = undef
  $kubelet_tls_private_key_file = undef
  $kubelet_volume_stats_agg_period = undef
  $kubelet_verbosity = 2
  $kubelet_extra_args = ''

  # proxy options
  $kube_proxy_service_ensure = 'running'
  $kube_proxy_journald_forward_enable = false
  $kube_proxy_service_enable = true
  $kube_proxy_manage_as = 'service'
  $kube_proxy_container_image = 'gcr.io/google_containers/hyperkube-amd64:v1.4.0'
  $kube_proxy_pod_cpu = '100m'
  $kube_proxy_pod_memory = '200Mi'
  $kube_proxy_bind_address = '127.0.0.1'
  $kube_proxy_cleanup_iptables = false
  $kube_proxy_cluster_cidr = undef
  $kube_proxy_config_sync_period = undef
  $kube_proxy_conntrack_max_per_core = undef
  $kube_proxy_conntrack_min = undef
  $kube_proxy_conntrack_tcp_timeout_close_wait = undef
  $kube_proxy_conntrack_tcp_timeout_established = undef
  $kube_proxy_google_json_key = undef
  $kube_proxy_healthz_bind_address = '127.0.0.1'
  $kube_proxy_healthz_port = 0
  $kube_proxy_hostname_override = undef
  $kube_proxy_iptables_masquerade_bit = undef
  $kube_proxy_iptables_min_sync_period = undef
  $kube_proxy_iptables_sync_period = undef
  $kube_proxy_kube_api_burst = undef
  $kube_proxy_kube_api_content_type = undef
  $kube_proxy_kube_api_qps = undef
  $kube_proxy_kubeconfig = undef
  $kube_proxy_masquerade_all = false
  $kube_proxy_master = undef
  $kube_proxy_oom_score_adj = undef
  $kube_proxy_proxy_mode = undef
  $kube_proxy_proxy_port_range = undef
  $kube_proxy_udp_timeout = undef
  $kube_proxy_verbosity = 2
  $kube_proxy_extra_args = ''
}
